mod commands;
mod handler;
mod util;

use serenity::all::{CommandInteraction, OnlineStatus};
use serenity::async_trait;
use serenity::builder::{CreateEmbed, CreateInteractionResponse, CreateInteractionResponseMessage};
use serenity::gateway::ActivityData;
use serenity::model::application::{Command, Interaction};
use serenity::model::gateway::Ready;
use serenity::prelude::*;
use util::{config, embed::Embed};

// This trait adds the `register_songbird` and `register_songbird_with` methods
// to the client builder below, making it easy to install this voice client.
// The voice client can be retrieved in any command using `songbird::get(ctx).await`.
use songbird::SerenityInit;

// YtDl requests need an HTTP client to operate -- we'll create and store our own.
use reqwest::Client as HttpClient;

struct HttpKey;
impl TypeMapKey for HttpKey {
    type Value = HttpClient;
}

struct Handler;

#[async_trait]
impl EventHandler for Handler {
    async fn interaction_create(&self, ctx: Context, interaction: Interaction) {
        if let Interaction::Command(command) = interaction {
            let content = Some(match command.data.name.as_str() {
                "info" => commands::info::run(&ctx, &command).await,
                "play" => commands::play::run(&ctx, &command).await,
                "stop" => commands::stop::run(&ctx, &command).await,
                _ => respond_with_error(&ctx, &command).await,
            });

            if let Some(embed) = content {
                let data = CreateInteractionResponseMessage::new().embed(embed);
                let builder = CreateInteractionResponse::Message(data);
                if let Err(why) = command.create_response(&ctx.http, builder).await {
                    println!("Cannot respond to slash command: {why}");
                }
            }
        }
    }

    async fn ready(&self, ctx: Context, ready: Ready) {
        println!("{} is connected!", ready.user.name);

        let _command = Command::create_global_command(&ctx.http, commands::info::register()).await;
        let _command = Command::create_global_command(&ctx.http, commands::stop::register()).await;
        let _command = Command::create_global_command(&ctx.http, commands::play::register()).await;

        println!("Created all public / commands");
    }
}

pub async fn respond_with_error(_ctx: &Context, command: &CommandInteraction) -> CreateEmbed {
    Embed::create(
        command.user.name.to_owned(),
        "Command not found",
        "Cannot find the executed command",
    )
}

#[tokio::main]
async fn main() {
    println!(
        r"__________               __                      .___
\______   \__ __ _______/  |_   ____    ____   __| _/____
 |       _/  |  |  ___/\   __\_/ __ \  /    \ / __ |/  _ \
 |    |   \  |  |___ \  |  |  \  ___/ |   |  | /_/ (  <_> )
 |____|_  /____/____  > |__|   \___  >|___|  |____ |\____/
        \/          \/             \/      \/     \/
"
    );

    // Load config
    let config = config::load().unwrap();
    // Set status
    let status = OnlineStatus::DoNotDisturb;
    let activity = ActivityData::streaming("music", "https://twitch.tv/moonleaytv").unwrap();

    // Build the client
    let mut client = Client::builder(config.discord_token, GatewayIntents::empty())
        .event_handler(Handler)
        .status(status)
        .activity(activity)
        .register_songbird()
        .type_map_insert::<HttpKey>(HttpClient::new())
        .await
        .expect("Error creating client");

    // Finally, start a single shard, and start listening to events.
    //
    // Shards will automatically attempt to reconnect, and will perform exponential backoff until
    // it reconnects.
    if let Err(why) = client.start().await {
        println!("Client error: {why:?}");
    }
}
